"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.integrityCheck = exports.listActionIdsAfter = exports.setAction = exports.getAction = exports.removeSnapshot = exports.setSnapshot = exports.getSnapshot = exports.listAllSnapshotIds = undefined;

var _regenerator = require("babel-runtime/regenerator");

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncIterator2 = require("babel-runtime/helpers/asyncIterator");

var _asyncIterator3 = _interopRequireDefault(_asyncIterator2);

var _asyncGenerator2 = require("babel-runtime/helpers/asyncGenerator");

var _asyncGenerator3 = _interopRequireDefault(_asyncGenerator2);

var _reverse2 = require("ramda/src/reverse");

var _reverse3 = _interopRequireDefault(_reverse2);

var _crypto = require("crypto");

var _crypto2 = _interopRequireDefault(_crypto);

var _jsonfile = require("jsonfile");

var _jsonfile2 = _interopRequireDefault(_jsonfile);

var _mkdirp = require("mkdirp");

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cacheFolder = process.env.JEC_JASK_CACHE_FOLDER || require("os").homedir() + "/.jecJaskCache";
var actionFolder = process.env.JEC_ACTIONS_FOLDER || require("os").homedir() + "/.jecActions";

var ensureActionFolderExists = function ensureActionFolderExists() {
	return new Promise(function (done, fail) {
		(0, _mkdirp2.default)(actionFolder, function (err) {
			return err ? fail(err) : done();
		});
	});
};

var hashString = function hashString(str) {
	return _crypto2.default.createHash("md5").update(str).digest("hex");
};

//------------------------------
//          snapshots
//------------------------------

var listAllSnapshotIds = exports.listAllSnapshotIds = function listAllSnapshotIds() {
	return new Promise(function (done, fail) {
		_fs2.default.readdir(cacheFolder, function (err, files) {
			return err ? fail(err) : done((0, _reverse3.default)(files) || []);
		});
	});
};

var getSnapshot = exports.getSnapshot = function getSnapshot(id) {
	return ensureActionFolderExists().then(function () {
		return new Promise(function (done, fail) {
			return _jsonfile2.default.readFile(cacheFolder + "/" + id, function (err, data) {
				return err ? fail(err) : done(data);
			});
		});
	});
};

var setSnapshot = exports.setSnapshot = function setSnapshot(id, snapshot) {
	return console.log({ id: id }), new Promise(function (done, fail) {
		_jsonfile2.default.writeFile(cacheFolder + "/" + id, snapshot, function (err) {
			return err ? fail(err) : done();
		});
	});
};

var removeSnapshot = exports.removeSnapshot = function removeSnapshot(id) {
	return new Promise(function (done, fail) {
		return _fs2.default.unlink(cacheFolder + "/" + id, function (err) {
			return err ? fail(err) : done();
		});
	});
};

//------------------------------
//           actions
//------------------------------

var getAction = exports.getAction = function getAction(id) {
	return ensureActionFolderExists().then(function () {
		return new Promise(function (done, fail) {
			_jsonfile2.default.readFile(actionFolder + "/" + id, function (err, dat) {
				return err ? fail(err) : done(dat);
			});
		});
	});
};

var setAction = exports.setAction = function setAction(action) {
	return ensureActionFolderExists().then(function () {
		return new Promise(function (done, fail) {
			_jsonfile2.default.writeFile(actionFolder + "/" + action.timestamp + "_" + action.type + "_" + action.salt, action, function (err) {
				return err ? fail(err) : done();
			});
		});
	});
};

var wrappedGetFiles = function wrappedGetFiles() {
	return ensureActionFolderExists().then(function () {
		return new Promise(function (done, fail) {
			_fs2.default.readdir(actionFolder, function (err, files) {
				return err ? fail(err) : done(files || []);
			});
		});
	});
};

var listActionIdsAfter = exports.listActionIdsAfter = function () {
	var _ref = _asyncGenerator3.default.wrap( /*#__PURE__*/_regenerator2.default.mark(function _callee(afterId) {
		var filenames, actions, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, _value2, id, integrity;

		return _regenerator2.default.wrap(function _callee$(_context) {
			while (1) {
				switch (_context.prev = _context.next) {
					case 0:
						_context.next = 2;
						return _asyncGenerator3.default.await(wrappedGetFiles());

					case 2:
						filenames = _context.sent;
						actions = filenames.reduce(function (acc, id) {
							return [].concat((0, _toConsumableArray3.default)(acc), [{
								id: id,
								integrity: integrityCheck(acc.slice(-1)[0], id)
							}]);
						}, []);
						_iteratorNormalCompletion = true;
						_didIteratorError = false;
						_iteratorError = undefined;
						_context.prev = 7;
						_iterator = (0, _asyncIterator3.default)(actions);

					case 9:
						_context.next = 11;
						return _asyncGenerator3.default.await(_iterator.next());

					case 11:
						_step = _context.sent;
						_iteratorNormalCompletion = _step.done;
						_context.next = 15;
						return _asyncGenerator3.default.await(_step.value);

					case 15:
						_value = _context.sent;

						if (_iteratorNormalCompletion) {
							_context.next = 24;
							break;
						}

						_value2 = _value, id = _value2.id, integrity = _value2.integrity;

						if (!(!afterId || afterId <= id)) {
							_context.next = 21;
							break;
						}

						_context.next = 21;
						return {
							id: id,
							integrity: integrity
						};

					case 21:
						_iteratorNormalCompletion = true;
						_context.next = 9;
						break;

					case 24:
						_context.next = 30;
						break;

					case 26:
						_context.prev = 26;
						_context.t0 = _context["catch"](7);
						_didIteratorError = true;
						_iteratorError = _context.t0;

					case 30:
						_context.prev = 30;
						_context.prev = 31;

						if (!(!_iteratorNormalCompletion && _iterator.return)) {
							_context.next = 35;
							break;
						}

						_context.next = 35;
						return _asyncGenerator3.default.await(_iterator.return());

					case 35:
						_context.prev = 35;

						if (!_didIteratorError) {
							_context.next = 38;
							break;
						}

						throw _iteratorError;

					case 38:
						return _context.finish(35);

					case 39:
						return _context.finish(30);

					case 40:
					case "end":
						return _context.stop();
				}
			}
		}, _callee, this, [[7, 26, 30, 40], [31,, 35, 39]]);
	}));

	function listActionIdsAfter(_x) {
		return _ref.apply(this, arguments);
	}

	return listActionIdsAfter;
}();

var integrityCheck = exports.integrityCheck = function integrityCheck() {
	var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
	    _ref2$id = _ref2.id,
	    id = _ref2$id === undefined ? "" : _ref2$id,
	    _ref2$integrity = _ref2.integrity,
	    integrity = _ref2$integrity === undefined ? "" : _ref2$integrity;

	var newAction = arguments[1];
	return hashString(id + integrity + newAction);
};